dialog = {
	show = function(caption, buttons, icon, defaultButton)
		
		local width = 290
		local leftEdge = 55
		local rightEdge = leftEdge + width
		
		local height = 172
		local topEdge = 34
		local bottomEdge = topEdge + height
		
		local captionPadding = 14
		local captionHeight = 82
		
		-- Outer border.
		screen.fillrect(leftEdge, topEdge, width, height, color.new(153, 204, 255))
		
		-- Main dialog face.
		screen.fillrect(leftEdge + 2, topEdge + 2, width - 4, height - 4, color.new(212, 212, 212))
		
		-- Shading around dialog face.
		screen.drawline(leftEdge + 2, topEdge + 2, rightEdge - 3, topEdge + 2, color.new(255, 255, 255))
		screen.drawline(leftEdge + 2, topEdge + 2, leftEdge + 2, bottomEdge - 3, color.new(255, 255, 255))
		screen.drawline(leftEdge + 2, bottomEdge - 3, rightEdge - 3, bottomEdge - 3, color.new(128, 128, 128))
		screen.drawline(rightEdge - 3, topEdge + 2, rightEdge - 3, bottomEdge - 3, color.new(128, 128, 128))
		
		-- Caption box.
		screen.fillrect(leftEdge + captionPadding, topEdge + captionPadding, width - captionPadding * 2, captionHeight, color.new(192, 192, 192))
		
		screen.drawline(leftEdge + captionPadding, topEdge + captionHeight + captionPadding - 1, rightEdge - captionPadding - 1, topEdge + captionHeight + captionPadding - 1, color.new(255, 255, 255))
		screen.drawline(rightEdge - captionPadding - 1, topEdge + captionPadding, rightEdge - captionPadding - 1, topEdge + captionHeight + captionPadding - 1, color.new(255, 255, 255))
		screen.drawline(leftEdge + captionPadding, topEdge + captionPadding, rightEdge - captionPadding - 1, topEdge + captionPadding, color.new(128, 128, 128))
		screen.drawline(leftEdge + captionPadding, topEdge + captionPadding, leftEdge + captionPadding, topEdge + captionHeight + captionPadding - 1, color.new(128, 128, 128))
		
		-- Divider.
		screen.drawline(leftEdge + 3, topEdge + captionHeight + captionPadding * 2, rightEdge - 4, topEdge + captionHeight + captionPadding * 2, color.new(128, 128, 128))
		screen.drawline(leftEdge + 3, topEdge + captionHeight + captionPadding * 2 + 1, rightEdge - 4, topEdge + captionHeight + captionPadding * 2 + 1, color.new(255, 255, 255))
		
		-- Icon.
		if icon then
			local icon = image.load("images/dialog/icons/" .. icon .. ".png")
			icon:draw(leftEdge + captionPadding + 13, topEdge + captionPadding + 11)
			icon:close()
			icon = nil
		end
		
		-- Caption.
		if caption then
			local caption = image.load("images/dialog/captions/" .. caption .. ".png")
			caption:draw(leftEdge + captionPadding + 71, topEdge + captionPadding + 10)
			caption:close()
			caption = nil
		end
		
		-- Determine the default button.
		if defaultButton == nil and #buttons > 0 then
			defaultButton = buttons[1]
		end
		
		-- Buttons.
		local buttonData = { }
		for buttonIndex = 1, #buttons do
			buttonData[buttonIndex] = {
				name = buttons[buttonIndex],
				x = math.floor(leftEdge + (width - (75 * #buttons)) / 2 + (buttonIndex - 1) * 75 + 6),
				y = bottomEdge - captionPadding - 38
			}
			local button = image.load("images/dialog/buttons/" .. buttonData[buttonIndex].name .. ".png")
			button:draw(buttonData[buttonIndex].x, buttonData[buttonIndex].y)
			button:close()
			button = nil
			if buttons[buttonIndex] == defaultButton then
				local buttonSelection = image.load("images/dialog/buttons/selection.png")
				buttonSelection:draw(buttonData[buttonIndex].x + 5, buttonData[buttonIndex].y + 5)
				buttonSelection:close()
				buttonSelection = nil
			end
		end
		
		if dialog.shown then
			dialog.shown()
		end
		
		screen.update()
		
		-- Wait for a button press.
		local result = nil
		while not result do
			
			-- Attempt to get input.
			if control.read() == 1 then
				if control.isButton() == 1 then
					if button.click() == 1 then
						result = defaultButton
					end
				elseif control.isTouch() == 1 then
					if touch.click() == 1 then
						x, y = touch.pos()
						if dialog.touchClick then
							dialog.touchClick(x, y)
						end
						for buttonIndex = 1, #buttonData do
							if x >= buttonData[buttonIndex].x and x < buttonData[buttonIndex].x + 63 and y >= buttonData[buttonIndex].y and y < buttonData[buttonIndex].y + 39 then
								-- Set the result.
								result = buttonData[buttonIndex].name
							end
						end
					elseif touch.down() == 1 then
						x, y = touch.pos()
						if dialog.touchDown then dialog.touchDown(x, y) end
					elseif touch.move() == 1 then
						x, y = touch.pos()
						if dialog.touchMove then dialog.touchMove(x, y) end
					elseif touch.up() == 1 then
						x, y = touch.pos()
						if dialog.touchUp then dialog.touchUp(x, y) end
					end
				end
			else
				os.sleep(10)
			end
		end

		-- Find the index of the pressed button.
		local resultIndex = 0
		for buttonIndex = 1, #buttonData do
			if buttonData[buttonIndex].name == result then
				resultIndex = buttonIndex
			end
		end
		
		-- Draw the pressed button graphic.
		if resultIndex > 0 then
			local button = image.load("images/dialog/buttons/" .. buttonData[resultIndex].name .. "-pressed.png")
			button:draw(buttonData[resultIndex].x, buttonData[resultIndex].y)
			button:close()
			button = nil
			if result == defaultButton then
				local buttonSelection = image.load("images/dialog/buttons/selection.png")
				buttonSelection:draw(buttonData[resultIndex].x + 7, buttonData[resultIndex].y + 7)
				buttonSelection:close()
				buttonSelection = nil
			end
			screen.update()
		end

		-- Make a noise.
		sounds.play("clck")
		-- Wait a bit.
		os.sleep(10)		

		-- Return the button press.
		return result
	end
}